#
import os

from keras.layers import Input, Dense, Conv1D, MaxPooling1D, GlobalAveragePooling1D, BatchNormalization, Flatten, Dropout, Bidirectional, LSTM, \
    concatenate
from keras.models import Model
from keras.models import Sequential
# from keras.utils.vis_utils import plot_model

from M1_CNN_model import con1D_2
import tensorflow as tf




# 1 Build Bi-LSTM

def bi_lstm(nb_lstm_outputs, nb_time_steps, nb_input_vectors):
    model = Sequential()
    model.add(Bidirectional(LSTM(
        units=nb_lstm_outputs,
        return_sequences=True),
        input_shape=(nb_time_steps, nb_input_vectors)))
    model.add(Bidirectional(LSTM(units=nb_lstm_outputs)))
    model.add(Dense(3, activation='softmax'))

    return model


def bi_lstm_input(input_shape, num_classes):
    x = Bidirectional(LSTM(
        units=77,
        return_sequences=True))(input_shape)

    x = Bidirectional(LSTM(
        units=71,
        return_sequences=False))(x)
    # x =Dense(300)(x)
    x = Dropout(0.2)(x)
    # x=Flatten()(x)
    x = Dense(num_classes * 3, activation='relu')(x)
    x1 = Dense(num_classes, activation='relu')(x)
    return x1


def merge_model(input_shape, output_cate_before_merge=51, num_classes=5):


    ls = bi_lstm_input(input_shape=input_shape, num_classes=output_cate_before_merge)

    model_cnn = con1D_2(input=input_shape, num_classes=output_cate_before_merge)
    # 4 model concat
    # concatenated = concatenate([ls,model_cnn ])
    # The 0.2 multiplied here is the weight I set myself, that is, only 20% of the fully connected network output is retained.
    concatenated = concatenate([ls * 0.2, model_cnn])


    x = Dense(num_classes * 10, activation='relu')(concatenated)
    x = Dense(num_classes * 3, activation='relu')(x)
    final_output = Dense(num_classes, activation='softmax')(x)
    return final_output
    # model_last = Model(inputs=[input_shape],outputs=final_output)
    # return model_last


def merge_model_v2(input_shape, num_classes=5):
    # 1.LSTM
    ls_1 = Bidirectional(LSTM(
        units=77,
        return_sequences=False))(input_shape)
    # x =Dense(300)(x)
    # ls_2 = Dropout(0.2)(ls_1)
    # x=Flatten()(x)
    # ls_3 = Dense(output_cate_before_merge, activation='relu')(ls_2)

    # 2. CNN
    x_cnn = Conv1D(
        filters=154,
        kernel_size=5,
        strides=1,
        padding='same',
        activation='relu',
        name='conv1'
    )(input_shape)
    x_cnn = GlobalAveragePooling1D()(x_cnn)
    # x_CNN=Dense(output_cate_before_merge,activation='softmax')(x_CNN)

    # 3. concat
    # concatenated = concatenate([ls_1 * 0.2, x_CNN_2])
    concatenated = concatenate([x_cnn, ls_1])
    x = Dropout(0.5)(concatenated)
    x = Dense(num_classes, activation='relu')(x)
    # final_output = Dense(num_classes, activation='softmax')(x)

    model_out = Model(inputs=input_shape, outputs=x)

    return model_out


def merge_model_v3(input_shape, num_classes=5):
    # 1.LSTM
    ls_1 = Bidirectional(LSTM(
        units=77,
        return_sequences=False))(input_shape)
    # x =Dense(300)(x)
    # ls_2 = Dropout(0.2)(ls_1)
    # x=Flatten()(x)
    # ls_3 = Dense(output_cate_before_merge, activation='relu')(ls_2)

    # 2. CNN
    x_cnn = Conv1D(
        # input_shape = input_shape,##input_shape
        filters=154,
        kernel_size=5,
        strides=1,
        padding='same',
        activation='relu',
        name='conv1'
    )(input_shape)
    x_cnn = GlobalAveragePooling1D()(x_cnn)
    # x_CNN=Dense(output_cate_before_merge,activation='softmax')(x_CNN)

    # concatenated = concatenate([ls_1 * 0.2, x_CNN_2])
    concatenated = concatenate([x_cnn, ls_1])
    # x = Dropout(0.5)(concatenated)
    x = Dense(200, activation='relu')(concatenated)
    x = Dense(num_classes, activation='relu')(x)
    # final_output = Dense(num_classes, activation='softmax')(x)

    model_out = Model(inputs=input_shape, outputs=x)

    return model_out


def merge_model_v4(input_shape, model_args, num_classes=5):
    # 1.LSTM
    ls_1 = Bidirectional(LSTM(
        units=77,
        return_sequences=False))(input_shape)
    # x =Dense(300)(x)
    # ls_2 = Dropout(0.2)(ls_1)
    # x=Flatten()(x)
    # ls_3 = Dense(output_cate_before_merge, activation='relu')(ls_2)

    # 2. CNN
    x_cnn = Conv1D(
        filters=154,
        kernel_size=5,
        strides=1,
        padding='same',
        activation='relu',
        name='conv1'
    )(input_shape)

    x_cnn = MaxPooling1D(
        pool_size=8,
        strides=2,
        padding='same',
        name='pool1')(x_cnn)
    if model_args['dropout'] > 0:
        x_cnn = Dropout(model_args['dropout'])(x_cnn)

    x_cnn = GlobalAveragePooling1D()(x_cnn)
    # x_CNN=Dense(output_cate_before_merge,activation='softmax')(x_CNN)

    # concatenated = concatenate([ls_1 * 0.2, x_CNN_2])
    concatenated = concatenate([x_cnn, ls_1])
    # x = Dropout(0.5)(concatenated)
    x = Dense(200, activation='relu')(concatenated)
    #
    if "dropout2" in model_args.keys():
        x = Dropout(model_args['dropout2'])(x)
    x = Dense(num_classes, activation='relu')(x)
    # final_output = Dense(num_classes, activation='softmax')(x)

    model_out = Model(inputs=input_shape, outputs=x)

    return model_out


def merge_model_v5(input_shape, model_args, num_classes=5):
    # 1.LSTM
    x = Bidirectional(LSTM(
        units=150,
        return_sequences=True))(input_shape)

    ls_1 = Bidirectional(LSTM(
        units=77,
        return_sequences=False))(x)
    # x =Dense(300)(x)
    # ls_2 = Dropout(0.2)(ls_1)
    # x=Flatten()(x)
    # ls_3 = Dense(output_cate_before_merge, activation='relu')(ls_2)

    # 2. CNN
    x_cnn = Conv1D(
        # input_shape = input_shape,##input_shape
        filters=154,
        kernel_size=5,
        strides=1,
        padding='same',
        activation='relu',
        name='conv1'
    )(input_shape)

    x_cnn = MaxPooling1D(
        pool_size=8,
        strides=2,
        padding='same',
        name='pool1')(x_cnn)
    if model_args['dropout'] > 0:
        x_cnn = Dropout(model_args['dropout'])(x_cnn)

    x_cnn = GlobalAveragePooling1D()(x_cnn)
    # x_CNN=Dense(output_cate_before_merge,activation='softmax')(x_CNN)

    # concatenated = concatenate([ls_1 * 0.2, x_CNN_2])
    concatenated = concatenate([x_cnn, ls_1])
    # x = Dropout(0.5)(concatenated)
    x = Dense(200, activation='relu')(concatenated)
    #
    if "dropout2" in model_args.keys():
        x = Dropout(model_args['dropout2'])(x)
    x = Dense(num_classes, activation='relu')(x)
    # final_output = Dense(num_classes, activation='softmax')(x)

    model_out = Model(inputs=input_shape, outputs=x)

    return model_out

def merge_model_v6(input_shape, model_args, num_classes=5):   #1DCNN is connected in series with BiLSTM, with 1DCNN at the front
    # 1. CNN
    x_cnn = Conv1D(
        filters=154,
        kernel_size=5,
        strides=1,
        padding='same',
        activation='relu',
        name='conv1'
    )(input_shape)

    x_cnn = MaxPooling1D(
        pool_size=8,
        strides=1,
        padding='same',
        name='pool1')(x_cnn)
    if model_args['dropout'] > 0:
        x_cnn = Dropout(model_args['dropout'])(x_cnn)

    # 2.LSTM
    ls_1 = Bidirectional(LSTM(
        units=77,
        return_sequences=False))(x_cnn)

    x = Dense(200, activation='relu')(ls_1)
    #
    if "dropout2" in model_args.keys():
        x = Dropout(model_args['dropout2'])(x)
    x = Dense(num_classes, activation='relu')(x)


    model_out = Model(inputs=input_shape, outputs=x)

    return model_out

def merge_model_v7(input_shape, model_args, num_classes=5):   #1DCNN is connected in series with BiLSTM, with 1DCNN at the back
    # 1.LSTM
    ls_1 = Bidirectional(LSTM(
        units=77,
        return_sequences=False))(input_shape)
    # x =Dense(300)(x)
    # ls_2 = Dropout(0.2)(ls_1)
    # x=Flatten()(x)
    # ls_3 = Dense(output_cate_before_merge, activation='relu')(ls_2)

    # ls_1 = GlobalAveragePooling1D()(ls_1)
    # x_CNN=Dense(output_cate_before_merge,activation='softmax')(x_CNN)

    x = tf.expand_dims(ls_1, axis=1)
   # 2. CNN
    x_cnn = Conv1D(
        filters=154,
        kernel_size=5,
        strides=1,
        padding='same',
        activation='relu',
        name='conv1'
    )(x)

    x_cnn = MaxPooling1D(
        pool_size=8,
        strides=2,
        padding='same',
        name='pool1')(x_cnn)
    if model_args['dropout'] > 0:
        x_cnn = Dropout(model_args['dropout'])(x_cnn)

    x_cnn = GlobalAveragePooling1D()(x_cnn)
    x = Dense(200, activation='relu')(x_cnn)
    #
    if "dropout2" in model_args.keys():
        x = Dropout(model_args['dropout2'])(x)
    x = Dense(num_classes, activation='relu')(x)
    # final_output = Dense(num_classes, activation='softmax')(x)

    model_out = Model(inputs=input_shape, outputs=x)

    return model_out


def alexnet1d(input_shape, num_classes=5):
    x = Conv1D(filters=96, kernel_size=11, strides=4, padding='same', activation='relu')(input_shape)
    x = BatchNormalization()(x)
    x = MaxPooling1D(pool_size=3, strides=2)(x)

    x = Conv1D(filters=256, kernel_size=5, padding='same', activation='relu')(x)
    x = BatchNormalization()(x)
    x = MaxPooling1D(pool_size=3, strides=2)(x)

    x = Conv1D(filters=384, kernel_size=3, padding='same', activation='relu')(x)
    x = BatchNormalization()(x)

    x = Conv1D(filters=384, kernel_size=3, padding='same', activation='relu')(x)
    x = BatchNormalization()(x)

    x = Conv1D(filters=256, kernel_size=3, padding='same', activation='relu')(x)
    x = BatchNormalization()(x)  # 批量归一化
    x = MaxPooling1D(pool_size=3, strides=2)(x)

    x = Flatten()(x)
    x = Dense(4096, activation='relu')(x)
    x = Dropout(0.5)(x)
    x = Dense(4096, activation='relu')(x)
    x = Dropout(0.5)(x)

    outputs = Dense(num_classes, activation='softmax')(x)

    model = Model(inputs=input_shape, outputs=outputs)
    return model


def residual_block(x, filters, kernel_size, stride=1, activation='relu'):

    y = Conv1D(filters, kernel_size=kernel_size, strides=stride, padding='same')(x)
    y = BatchNormalization()(y)
    y = Activation(activation)(y)

    y = Conv1D(filters, kernel_size=kernel_size, strides=1, padding='same')(y)
    y = BatchNormalization()(y)

    # concat
    if x.shape[-1] != filters:
        x = Conv1D(filters, kernel_size=1, strides=stride, padding='same')(x)

    out = Add()([x, y])
    out = Activation(activation)(out)
    return out



def resnet(input_shape, model_args, num_classes=5):
    # Initial convolutional layer
    x = Conv1D(filters=64, kernel_size=7, strides=2, padding='same')(input_shape)
    x = BatchNormalization()(x)
    x = Activation('relu')(x)

    # Applying residual blocks
    x = residual_block(x, filters=64, kernel_size=3)
    x = residual_block(x, filters=64, kernel_size=3)

    x = MaxPooling1D(pool_size=3, strides=2, padding='same')(x)

    x = residual_block(x, filters=128, kernel_size=3,stride=2)
    x = residual_block(x, filters=128, kernel_size=3)

    x = MaxPooling1D(pool_size=3, strides=2, padding='same')(x)

    x = residual_block(x, filters=256, kernel_size=3,stride=2)
    x = residual_block(x, filters=256, kernel_size=3)

    x = MaxPooling1D(pool_size=3, strides=2, padding='same')(x)

    x = residual_block(x, filters=512, kernel_size=3,stride=2)
    x = residual_block(x, filters=512,kernel_size=3)

    x = GlobalAveragePooling1D()(x)

    # Fully connected layers
    x = Dense(200, activation='relu')(x)
    if "dropout2" in model_args.keys():
        x = Dropout(model_args['dropout2'])(x)

    outputs = Dense(num_classes, activation='softmax')(x)

    # Creating model
    model = Model(inputs=input_shape, outputs=outputs)

    return model

def conv1d(input_shape, model_args, num_classes=5):

    # 2. CNN
    x_cnn = Conv1D(
        filters=154,
        kernel_size=5,
        strides=1,
        padding='same',
        activation='relu',
        name='conv1'
    )(input_shape)

    x_cnn = MaxPooling1D(
        pool_size=8,
        strides=2,
        padding='same',
        name='pool1')(x_cnn)
    if model_args['dropout'] > 0:
        x_cnn = Dropout(model_args['dropout'])(x_cnn)

    x_cnn = GlobalAveragePooling1D()(x_cnn)
    # x_CNN=Dense(output_cate_before_merge,activation='softmax')(x_CNN)
    # 3. concat
    # concatenated = concatenate([ls_1 * 0.2, x_CNN_2])
    # x = Dropout(0.5)(concatenated)
    x = Dense(200, activation='relu')(x_cnn)
    #
    if "dropout2" in model_args.keys():
        x = Dropout(model_args['dropout2'])(x)
    x = Dense(num_classes, activation='relu')(x)
    # final_output = Dense(num_classes, activation='softmax')(x)

    model_out = Model(inputs=input_shape, outputs=x)

    return model_out

def lstm(input_shape, model_args, num_classes=5):
    # 1.LSTM
    x = LSTM(units=154,
        return_sequences=False)(input_shape)

    # x =Dense(300)(x)
    # ls_2 = Dropout(0.2)(ls_1)
    # x=Flatten()(x)
    # ls_3 = Dense(output_cate_before_merge, activation='relu')(ls_2)

    # x_CNN=Dense(output_cate_before_merge,activation='softmax')(x_CNN)

    # concatenated = concatenate([ls_1 * 0.2, x_CNN_2])
    # x = Dropout(0.5)(concatenated)
    x = Dense(200, activation='relu')(x)
    #
    if "dropout2" in model_args.keys():
        x = Dropout(model_args['dropout2'])(x)
    x = Dense(num_classes, activation='relu')(x)
    # final_output = Dense(num_classes, activation='softmax')(x)

    model_out = Model(inputs=input_shape, outputs=x)

    return model_out


def bilstm(input_shape, model_args, num_classes=5):
    # 1.LSTM
    x = Bidirectional(LSTM(
        units=77,
        return_sequences=False))(input_shape)

    # x =Dense(300)(x)
    # ls_2 = Dropout(0.2)(ls_1)
    # x=Flatten()(x)
    # ls_3 = Dense(output_cate_before_merge, activation='relu')(ls_2)


    # x_CNN=Dense(output_cate_before_merge,activation='softmax')(x_CNN)
    # concatenated = concatenate([ls_1 * 0.2, x_CNN_2])
    # x = Dropout(0.5)(concatenated)
    x = Dense(200, activation='relu')(x)
    #
    if "dropout2" in model_args.keys():
        x = Dropout(model_args['dropout2'])(x)
    x = Dense(num_classes, activation='relu')(x)
    # final_output = Dense(num_classes, activation='softmax')(x)

    model_out = Model(inputs=input_shape, outputs=x)

    return model_out


def conv1d_lstm(input_shape, model_args, num_classes=5):
    # 1.LSTM
    ls_1 = LSTM(
        units=154,
        return_sequences=False)(input_shape)
    # x =Dense(300)(x)
    # ls_2 = Dropout(0.2)(ls_1)
    # x=Flatten()(x)
    # ls_3 = Dense(output_cate_before_merge, activation='relu')(ls_2)

    # 2. CNN
    x_cnn = Conv1D(

        filters=154,
        kernel_size=5,
        strides=1,
        padding='same',
        activation='relu',
        name='conv1'
    )(input_shape)

    x_cnn = MaxPooling1D(
        pool_size=8,
        strides=2,
        padding='same',
        name='pool1')(x_cnn)
    if model_args['dropout'] > 0:
        x_cnn = Dropout(model_args['dropout'])(x_cnn)

    x_cnn = GlobalAveragePooling1D()(x_cnn)
    # x_CNN=Dense(output_cate_before_merge,activation='softmax')(x_CNN)

    # concatenated = concatenate([ls_1 * 0.2, x_CNN_2])
    concatenated = concatenate([x_cnn, ls_1])
    # x = Dropout(0.5)(concatenated)
    x = Dense(200, activation='relu')(concatenated)
    #
    if "dropout2" in model_args.keys():
        x = Dropout(model_args['dropout2'])(x)
    x = Dense(num_classes, activation='relu')(x)
    # final_output = Dense(num_classes, activation='softmax')(x)

    model_out = Model(inputs=input_shape, outputs=x)

    return model_out


def bilstm_layer3(input_shape, model_args, num_classes=5):
    # 1.LSTM
    x = Bidirectional(LSTM(
        units=200,
        return_sequences=True))(input_shape)

    ls_1 = Bidirectional(LSTM(
        units=150,
        return_sequences=True))(x)

    ls_2 = Bidirectional(LSTM(
        units=77,
        return_sequences=False))(ls_1)
    # x =Dense(300)(x)
    # ls_2 = Dropout(0.2)(ls_1)
    # x=Flatten()(x)
    # ls_3 = Dense(output_cate_before_merge, activation='relu')(ls_2)

    # 2. CNN
    x_cnn = Conv1D(
        filters=154,
        kernel_size=5,
        strides=1,
        padding='same',
        activation='relu',
        name='conv1'
    )(input_shape)

    x_cnn = MaxPooling1D(
        pool_size=8,
        strides=2,
        padding='same',
        name='pool1')(x_cnn)
    if model_args['dropout'] > 0:
        x_cnn = Dropout(model_args['dropout'])(x_cnn)

    x_cnn = GlobalAveragePooling1D()(x_cnn)
    # x_CNN=Dense(output_cate_before_merge,activation='softmax')(x_CNN)

    # concatenated = concatenate([ls_1 * 0.2, x_CNN_2])
    concatenated = concatenate([x_cnn, ls_2])
    # x = Dropout(0.5)(concatenated)
    x = Dense(200, activation='relu')(concatenated)
    #
    if "dropout2" in model_args.keys():
        x = Dropout(model_args['dropout2'])(x)
    x = Dense(num_classes, activation='relu')(x)
    # final_output = Dense(num_classes, activation='softmax')(x)

    model_out = Model(inputs=input_shape, outputs=x)

    return model_out

def bilstm_layer2(input_shape, model_args, num_classes=5):
    # 1.LSTM
    x = Bidirectional(LSTM(
        units=150,
        return_sequences=True))(input_shape)

    ls_1 = Bidirectional(LSTM(
        units=77,
        return_sequences=False))(x)
    # x =Dense(300)(x)
    # ls_2 = Dropout(0.2)(ls_1)
    # x=Flatten()(x)
    # ls_3 = Dense(output_cate_before_merge, activation='relu')(ls_2)

    # 2. CNN
    x_cnn = Conv1D(
        filters=154,
        kernel_size=5,
        strides=1,
        padding='same',
        activation='relu',
        name='conv1'
    )(input_shape)

    x_cnn = MaxPooling1D(
        pool_size=8,
        strides=2,
        padding='same',
        name='pool1')(x_cnn)
    if model_args['dropout'] > 0:
        x_cnn = Dropout(model_args['dropout'])(x_cnn)

    x_cnn = GlobalAveragePooling1D()(x_cnn)
    # x_CNN=Dense(output_cate_before_merge,activation='softmax')(x_CNN)

    # concatenated = concatenate([ls_1 * 0.2, x_CNN_2])
    concatenated = concatenate([x_cnn, ls_1])
    # x = Dropout(0.5)(concatenated)
    x = Dense(200, activation='relu')(concatenated)
    #
    if "dropout2" in model_args.keys():
        x = Dropout(model_args['dropout2'])(x)
    x = Dense(num_classes, activation='relu')(x)
    # final_output = Dense(num_classes, activation='softmax')(x)

    model_out = Model(inputs=input_shape, outputs=x)

    return model_out




def dropout_3(input_shape, model_args, num_classes=5):
    # 1.LSTM
    ls_1 = Bidirectional(LSTM(
        units=77,
        return_sequences=False))(input_shape)
    # x =Dense(300)(x)
    # ls_2 = Dropout(0.2)(ls_1)
    # x=Flatten()(x)
    # ls_3 = Dense(output_cate_before_merge, activation='relu')(ls_2)

    # 2. CNN
    x_cnn = Conv1D(
        filters=154,
        kernel_size=5,
        strides=1,
        padding='same',
        activation='relu',
        name='conv1'
    )(input_shape)

    x_cnn = MaxPooling1D(
        pool_size=8,
        strides=2,
        padding='same',
        name='pool1')(x_cnn)
    if model_args['dropout3'] > 0:
        x_cnn = Dropout(model_args['dropout3'])(x_cnn)

    x_cnn = GlobalAveragePooling1D()(x_cnn)
    # x_CNN=Dense(output_cate_before_merge,activation='softmax')(x_CNN)

    # concatenated = concatenate([ls_1 * 0.2, x_CNN_2])
    concatenated = concatenate([x_cnn, ls_1])
    # x = Dropout(0.5)(concatenated)
    x = Dense(200, activation='relu')(concatenated)
    #
    if "dropout3" in model_args.keys():
        x = Dropout(model_args['dropout3'])(x)
    x = Dense(num_classes, activation='relu')(x)
    # final_output = Dense(num_classes, activation='softmax')(x)

    model_out = Model(inputs=input_shape, outputs=x)

    return model_out

def dropout_4(input_shape, model_args, num_classes=5):
    # 1.LSTM
    ls_1 = Bidirectional(LSTM(
        units=77,
        return_sequences=False))(input_shape)
    # x =Dense(300)(x)
    # ls_2 = Dropout(0.2)(ls_1)
    # x=Flatten()(x)
    # ls_3 = Dense(output_cate_before_merge, activation='relu')(ls_2)

    # 2. CNN
    x_cnn = Conv1D(
        filters=154,
        kernel_size=5,
        strides=1,
        padding='same',
        activation='relu',
        name='conv1'
    )(input_shape)

    x_cnn = MaxPooling1D(
        pool_size=8,
        strides=2,
        padding='same',
        name='pool1')(x_cnn)
    if model_args['dropout4'] > 0:
        x_cnn = Dropout(model_args['dropout4'])(x_cnn)

    x_cnn = GlobalAveragePooling1D()(x_cnn)
    # x_CNN=Dense(output_cate_before_merge,activation='softmax')(x_CNN)

    # concatenated = concatenate([ls_1 * 0.2, x_CNN_2])
    concatenated = concatenate([x_cnn, ls_1])
    # x = Dropout(0.5)(concatenated)
    x = Dense(200, activation='relu')(concatenated)
    #
    if "dropout4" in model_args.keys():
        x = Dropout(model_args['dropout4'])(x)
    x = Dense(num_classes, activation='relu')(x)
    # final_output = Dense(num_classes, activation='softmax')(x)

    model_out = Model(inputs=input_shape, outputs=x)

    return model_out


if __name__ == '__main__':

    import m0_para


    model_pic_save = m0_para.model_pic_save
    model_last = merge_model_v5(input_shape=Input(shape=(200, 68)),
                                 num_classes=6,
                                 model_args={'dropout': 0.2,"dropout2":0.2})
    model_last.summary()

    # plot_model(model_last,
    #            to_file=model_pic_save + os.sep + "last_moedl.png",
    #            show_shapes=True,
    #            show_layer_names='False',
    #            rankdir='TB')

