# -*- coding: UTF-8 -*-
# #

import numpy as np
from tensorflow.keras.utils import to_categorical

from M4_get_features import get_feature_by_pyAudio
from ThreadingSafetyIter import threadsafe_generator


# model.fit_generator
class random_uniform_num():
    """
    Uniformly random, ensuring each appears only once in each round
    Specify batchsize to ensure that all samples (total) will not be repeated in a round
    """

    def __init__(self, total):
        self.total = total
        self.ranges = [i for i in range(total)]
        np.random.shuffle(self.ranges)
        self.index = 0

    # @property
    # def ranges(self):
    #     return np.random.shuffle([i for i in range(self.total)])

    def get(self, batchsize):
        r_n = []
        if (self.index + batchsize > self.total):
            r_n_1 = self.ranges[self.index:self.total]
            np.random.shuffle(self.ranges)
            self.index = (self.index + batchsize) - self.total
            r_n_2 = self.ranges[0:self.index]
            r_n.extend(r_n_1)
            r_n.extend(r_n_2)
        else:
            r_n = self.ranges[self.index: self.index + batchsize]
            self.index = self.index + batchsize
        return r_n

@threadsafe_generator

def data_generator_pyAudio(mark_data, batch_size, types_num, feature_args):
    """
    The purpose is to randomly select a batch of data to achieve the purpose of disrupting the data. Iteratively output data to the model.
     pyAudio was used to extract audio features. 68 audio features were obtained.
     The output data size is (max_pad_size, number of features, 1)
    """

    # For the first dimension of each sample data, divide the maximum original data size by the number of adjacent columns that need to be integrated, and take ceil to get the final data length.
    # The default use_data_len is False, which means taking all data
    # sample_shape = use_data_len if use_data_len else len(mark_data) # If use_data_len is passed in, it means that only a certain amount of data is taken for testing.

    file_path = feature_args['file_path']
    input_data_shape = feature_args['input_data_shape']
    features_position_list = feature_args['feature_position_list']
    win = feature_args['win']
    step = feature_args['step']
    audio_use_seconds = feature_args['audio_use_seconds']
    max_pad_size = feature_args['max_pad_size']
    use_all_col=feature_args['use_all_col']
    if use_all_col:
        mean_col_num=False
    else:
        mean_col_num = max_pad_size / input_data_shape
    start_seconds = feature_args['start_seconds']
    save_as_pickle = feature_args['save_as_pickle']
    get_max = feature_args['get_max']
    label_col_name = feature_args['label_col_name']

    # Instantiate
    r_n = random_uniform_num(len(mark_data))
    _audiofile = np.array(list(mark_data[file_path]))
    while 1:
        x = np.zeros((batch_size, input_data_shape, len(features_position_list)), dtype=np.float64)
        y = np.zeros((batch_size, types_num), dtype=np.float64)
        shufaudiofile = _audiofile[r_n.get(batch_size)]
        # print(shufaudiofile)
        for i, j in enumerate(shufaudiofile):

            feactures, _ = get_feature_by_pyAudio(file_in=j,
                                                  mean_col_num=mean_col_num,
                                                  input_data_shape=input_data_shape,
                                                  win=win,
                                                  step=step,
                                                  max_pad_size=max_pad_size,
                                                  use_seconds=audio_use_seconds,
                                                  start_seconds=start_seconds,
                                                  save_as_pickle=save_as_pickle,
                                                  get_max=get_max)
            features_use = feactures[:, features_position_list]
            if features_use.shape != (input_data_shape, len(features_position_list)):
                print(
                    f"When reading data, input_data_shape should be{input_data_shape}:{len(features_position_list)}, in reality:{features_use.shape}")
                continue
            x[i] = np.array(features_use)
            # Take out the corresponding file, learn the corresponding tone score (0-5), and perform hot encoding
            var_out = list(mark_data.loc[mark_data[file_path] == j][label_col_name])[0]
            y[i] = to_categorical(var_out, num_classes=types_num)
            # print(var_out,y[i])
            # reshape
            # feactures=feactures.reshape(feactures.shape[0], feactures.shape[1], 1)

        print("\n After data sorting, the shape of x: {}; the shape of y: : {}".format(x.shape, y.shape))
        yield (x, y)
