# -*- coding: UTF-8 -*-

# Use pyAudioAnalysis to quickly extract 68 features

import math
import os
import pickle

import librosa
import numpy as np
from pyAudioAnalysis import MidTermFeatures as mF
from pyAudioAnalysis import ShortTermFeatures as aF
from pyAudioAnalysis import audioBasicIO as aIO




def get_near(df, input_data_shape, mean_col_num, get_max=False):
    """
    This is to take the average value of adjacent columns. In fact, you can also consider taking the maximum value of adjacent columns.
    """
    # df=f.copy()
    # near_num=3
    list_get = []
    mean_col_num = math.ceil(mean_col_num)
    for i in range(input_data_shape):
        begin = i * mean_col_num
        # print([begin, begin + 3])
        new = df[:, begin:begin + mean_col_num]
        if get_max:
            new = np.max(new, axis=1)
        else:
            new = np.mean(new, axis=1)
        list_get.append(new)
    out = np.array(list_get)
    # print(out.shape)
    return out



def get_feature_by_pyAudio(file_in,
                           mean_col_num,
                           input_data_shape,
                           win,
                           step,
                           max_pad_size=False,
                           use_seconds=15,
                           start_seconds=0,
                           save_as_pickle=False,
                           get_max=False
                           ):
    # Determine whether the file exists
    save_file = file_in.replace('.wav', f'{start_seconds}_{use_seconds}.pickle')
    if os.path.exists(save_file):
        print('file exists')

        return pickle.load(open(save_file, 'rb'))
    # 1. Read audio files
    # fs refers to the 16kHz mentioned at the beginning of the article, which is 16,000 samples per second, and s refers to the total number of samples in your audio file
    fs, s = aIO.read_audio_file(file_in)

    s = s[start_seconds * fs: use_seconds * fs]

    # Extract short-term signals, win and step represent the window and step size, /
    # f is a matrix of [68, segment length], and fn is the feature name
    [f, fn] = aF.feature_extraction(s, fs, int(fs * win), int(fs * step))
    # print(f'{f.shape[1]} frames, {f.shape[0]} short-term features')
    # print('Feature names:')
    # for i, nam in enumerate(fn):
    #     print(f'{i}:{nam}')
    y_shape = f.shape[1]

    if max_pad_size:
        if y_shape < max_pad_size:
            pad_size = max_pad_size - y_shape
            f = np.pad(f, ((0, 0), (pad_size, 0)), mode='constant')
        else:
            f = f[:, :max_pad_size]

    if mean_col_num:
        f = get_near(df=f,
                     input_data_shape=input_data_shape,
                     mean_col_num=mean_col_num,
                     get_max=get_max)
    else:
        f=f.T

    # print(f.shape)

    if save_as_pickle:
        with open(save_file, 'wb') as file:
            pickle.dump((f, fn), file)

    return f, fn


# def get_feature_by_pyAudio_mul_seg(file_in,
#                                    mean_col_num,
#                                    input_data_shape,
#                                    win,
#                                    step,
#                                    max_pad_size=False,
#                                    use_seconds=15,
#                                    get_max=False
#                                    ):
#
#     fs, s = aIO.read_audio_file(file_in)
#     # print(fs, s.shape)
#     len_wav = len(s) / fs
#
#     segs = round(len_wav / use_seconds)
#
#     data_out = np.zeros((segs, input_data_shape, 68), dtype=np.float)
#
#     for i in range(segs):
#         # print([i * use_seconds * fs, (i + 1) * use_seconds * fs])
#         s_seg_i = s[i * use_seconds * fs:(i + 1) * use_seconds * fs]
#
#
#         [f, fn] = aF.feature_extraction(s_seg_i, fs, int(fs * win), int(fs * step))
#         # print(f'{f.shape[1]} frames, {f.shape[0]} short-term features')
#         # print('Feature names:')
#         # for i, nam in enumerate(fn):
#         #     print(f'{i}:{nam}')
#         y_shape = f.shape[1]
#
#         if max_pad_size:
#             if y_shape < max_pad_size:
#                 pad_size = max_pad_size - y_shape
#                 f = np.pad(f, ((0, 0), (pad_size, 0)), mode='constant')
#             else:
#                 f = f[:, :max_pad_size]
#
#         if mean_col_num>1:
#             f = get_near(df=f,
#                          input_data_shape=input_data_shape,
#                          mean_col_num=mean_col_num,
#                          get_max=get_max)
#         else:
#             f=f.T
#         data_out[i] = f
#     # print(f.shape)
#     return data_out,len_wav,segs


#
# def get_mfccs(wavfile, max_pad_size=False):
#     X, sample_rate = librosa.load(path=wavfile, sr=16000, mono=True)  #
#     mfccs = librosa.feature.mfcc(y=X, sr=sample_rate)
#     mfccs = np.array(mfccs.T)  # 转置一下(56,20)
#     y_shape = mfccs.shape[0]
#     if max_pad_size:
#         if y_shape < max_pad_size:
#             pad_size = max_pad_size - y_shape
#             mfccs = np.pad(mfccs, ((pad_size, 0), (0, 0)), mode='constant')
#         else:
#             mfccs = mfccs[:max_pad_size, :]
#         return mfccs  # (max_pad_size,20)
#
#     else:
#         return mfccs


# if __name__ == '__main__':

    # file_in = r'D:\hc--document\all text\P8_medical\thesis_dataset\use_wav_raw'
    #
    # input_data_shape = 200
    # win = 0.050
    # step = 0.025
    # max_pad_size = 200
    # use_seconds = 15
    # start_seconds = 0
    # mean_col_num = max_pad_size / input_data_shape
    # get_max = False
    # fs, s = aIO.read_audio_file(file_in)
    # s = s[start_seconds * fs: use_seconds * fs]
    #
    #
    # [f, fn] = aF.feature_extraction(s, fs, int(fs * win), int(fs * step))
    # print(f'{f.shape[1]} frames, {f.shape[0]} short-term features')
    # print('Feature names:')
    # for i, nam in enumerate(fn):
    #     print(f'{i}:{nam}')
    #
    #
    # f, fn = get_feature_by_pyAudio(file_in, mean_col_num, input_data_shape, win, step, max_pad_size,
    #                                use_seconds, start_seconds)
    #
    # f = get_feature_by_pyAudio_mul_seg(file_in, mean_col_num,
    #                                    input_data_shape, win, step,
    #                                    max_pad_size,
    #                                    use_seconds,
    #                                    get_max)
    #
    # import pandas as pd
    #
    # f2 = pd.DataFrame(f)
    # b = f2[[0, 1, 34, 35]]
    # path_file2 = r'D:\hc--document\all text\P8_medical\thesis_dataset\use_wav_raw'
    # fn2, f2 = get_feature_by_pyAudio(path_file2)
    #
    # dd = get_mfccs(wavfile=file_in, max_pad_size=False)
    # dd.shape  # (157, 20)
