# -*- coding: UTF-8 -*-
#
import datetime
import json
import os
import pickle
import socket
# import sys
# sys.path.append(os.getcwd())
import sys

import numpy as np
import tensorflow as tf
from keras.callbacks import ReduceLROnPlateau, EarlyStopping, ModelCheckpoint, LearningRateScheduler, TensorBoard, \
    Callback
from keras.layers import Input
from keras.models import load_model

sys.path.append(os.getcwd())
sys.path.append(r'D:\hc--document\all text\P7_2_tone')   # Replace with your file path
from M3_load_data_utf8 import data_generator_pyAudio
import M2_ConvLSTM, m0_para


def save_as_ob(path, save, t=''):
    if save == True:
        with open(path, 'wb') as p:
            ob = pickle.dump(t, p)  #
    else:
        with open(path, 'rb+') as r:  # ,encoding='iso-8859-1'
            ob = pickle.load(r)  #
    return ob


def Write_Text(file_name, contant):
    # file_name = 'test.txt'
    with open(file_name, "a+", encoding='utf-8') as f:
        f.writelines(str(contant))
        f.writelines("\n")
        # print(contant)


def get_session(gpu_fraction=0.9):
    num_threads = os.environ.get('OMP_NUM_THREADS')
    gpu_options = tf.compat.v1.GPUOptions(per_process_gpu_memory_fraction=gpu_fraction)

    if num_threads:
        return tf.compat.v1.Session(
            config=tf.compat.v1.ConfigProto(gpu_options=gpu_options, intra_op_parallelism_threads=num_threads))
    else:
        return tf.compat.v1.Session(config=tf.compat.v1.ConfigProto(gpu_options=gpu_options, allow_soft_placement=True))


def get_least_file(path):
    # Get all the files (names) in the folder and return them in list form
    lists = os.listdir(path)
    lists_use = [i for i in lists if os.path.basename(i).split(".")[-1] == 'h5']
    lists_use = [i for i in lists_use if "nan" not in i]

    # print("Unprocessed folder list：\n %s \n"%lists )

    # Arrange in raw order according to the keywords of the key. Lambda enters parameter x as an element of the lists list to obtain the last modification date of the file.
    # Finally sort the lists by file time from small to large
    lists_use.sort(key=lambda x: os.path.getmtime((path + os.sep + x)))

    # Get the absolute path of the latest file, the last value in the list, folder + file name
    file_new = os.path.join(path, lists_use[-1])
    return file_new


def train(use_model,
          path_wav_train, trian_file, test_file,
          fit_pars, feature_args,
          ):
    """

    :param x_size: Number of samples
    :param num_epochs: How many rounds of data cycle
    :param max_pad_size: Data length (such as audio length)
    :param feature_num: Number of features
    :param types_num: Number of label categories
    :param batch_size: The number of samples passed in each time
    :param use_model: What model to use
    :return:
    """
    # 1. The location where the relevant data of the model is saved
    version_name = fit_pars['version_name']
    args_info_save = fit_pars['args_info_save']
    save_dir = os.path.join(path_wav_train, version_name, 's2_run_record')
    if not os.path.exists(save_dir): os.makedirs(save_dir)
    save_dir_log = os.path.join(save_dir, 'logs')

    # K.set_session(get_session())
    with open(save_dir + os.sep + args_info_save, 'a+') as f:
        f.write("\n\n\n")
        f.write(json.dumps(fit_pars))
        f.write("\n")
        f.write(json.dumps(feature_args))
        f.write("\n")

    # 2. Relevant parameter extraction
    feature_num = len(feature_args['feature_position_list'])
    input_data_shape = feature_args['input_data_shape']
    num_epochs = fit_pars['num_epochs']
    num_classes = fit_pars['types_num']
    loss = fit_pars['loss']
    optimizer = fit_pars['optimizer']
    metrics = fit_pars['metrics']
    loadModel = fit_pars['load_ex_modol']
    feature_args['audio_use_seconds'] = fit_pars['audio_use_seconds']
    feature_args['max_pad_size'] = fit_pars['max_pad_size']
    learning_rate_mode = fit_pars['learning_rate_mode']
    learning_rate_dict = fit_pars['learning_rate_dict']
    model_args = fit_pars['model_args']
    # 3) Import or create a new model

    try:
        # Determine which models are available based on their names
        # save_dir=r'E:\data_backup\P8_medical\P1_big5personality\self_model\saved_models\neuroticism\run_record'
        model_path = get_least_file(save_dir)
        # if '/' in os.getcwd():
        #     pass
        #     model_path = glob.glob(save_dir + "/*.h5")[-1]
        # else:
        #     model_path = glob.glob(save_dir + "\*.h5")[-1]
        # Load previous model
        print(">>>>>>Import existing model {}".format(model_path))
        model = load_model(model_path)
        with open(save_dir + os.sep + args_info_save, 'a+') as f:
            f.write(model_path)
            f.write("\n")
        # model.summary()
    except:
        print(">>>>>>No models were imported")

        # (1) Define model
        # input = Input(shape=(max_pad_size, feature_num, 1), name='the_input')
        input = Input(shape=(input_data_shape, feature_num), name='the_input')
        model = use_model(input_shape=input, num_classes=num_classes, model_args=model_args)
        # model = multi_gpu_model(model, 4)  # Multi-GPU training

        # (2) Compile the model and specify the loss function
        model.compile(loss=loss, optimizer=optimizer, metrics=metrics)
        # model.summary()
        # print('build model done ...')

    # 3) Introduce data iterator
    train_size = trian_file.shape[0]
    validation_size = test_file.shape[0]
    print("train dataset size: {}; test dataset size: : {}".format(trian_file.shape, test_file.shape))
    # Training data set (x,y)
    train_loader = data_generator_pyAudio(mark_data=trian_file,
                                          batch_size=fit_pars['batch_size'],
                                          types_num=fit_pars['types_num'],
                                          feature_args=feature_args,
                                          )
    # Test set (x,y)
    test_loader = data_generator_pyAudio(mark_data=test_file,
                                         batch_size=fit_pars['batch_size'],
                                         types_num=fit_pars['types_num'],
                                         feature_args=feature_args
                                         )
    # 4)Specify a callback function that can automatically save the model
    # filepath = "model_{epoch:02d}-{loss:.2f}.h5"
    t = datetime.datetime.now().strftime('%Y_%m_%d_%H_%M_%S')
    # datetime.datetime.strptime(t,'%Y_%m_%d_%H_%M_%S')
    filepath = "model_" + t + "_{epoch:02d}-{val_loss:.2f}.h5"
    # filepath = "model_{epoch:02d}-{val_accuracy:.4f}.h5"
    filepath = os.path.join(save_dir, filepath)
    checkpoint = ModelCheckpoint(filepath=filepath,  # monitor='val_loss',
                                 save_best_only=False,
                                 # save_best_only=True,
                                 save_weights_only=False,
                                 save_freq='epoch')

    # 5) Specify earlystop
    earlystop = EarlyStopping(monitor='val_loss', patience=3000, verbose=1)

    # 6) Specify visualization
    tensorboard = TensorBoard(log_dir=save_dir_log, write_graph=True)

    # 7) Customize a callback function to record the results of each epoch
    class infoHistory(Callback):
        def on_epoch_end(self, epoch, logs={}):
            with open(save_dir + os.sep + 'log_epoch.txt', 'a+') as f:
                f.write(str(epoch) + ":" + str(logs) + "\n")

    # history = infoHistory()

    if learning_rate_mode == 1:
        if learning_rate_dict:
            keys = list(learning_rate_dict.keys())
            values = list(learning_rate_dict.values())
            learning_rate = np.array(([0.001] * num_epochs))
            for i in range(num_epochs):
                if (i < keys[0]):
                    learning_rate[i] = values[0]
                elif i < keys[1]:
                    learning_rate[i] = values[1]
                elif i < keys[2]:
                    learning_rate[i] = values[2]
                else:
                    learning_rate[i] = 0.00001
        else:
            learning_rate = np.array(([0.001] * num_epochs))
            for i in range(num_epochs):
                if (i < 100):
                    learning_rate[i] = 0.01
                elif i < 200:
                    learning_rate[i] = 0.005
                elif i < 300:
                    learning_rate[i] = 0.0001
                else:
                    learning_rate[i] = 0.00001
        Write_Text(save_dir + os.sep + args_info_save, learning_rate[0:200])
        changelr = LearningRateScheduler(lambda epoch: float(learning_rate[epoch]))
        callbacks = [checkpoint, earlystop, changelr,tensorboard]# history,
        print(learning_rate[0:200])
        # adam = optimizers.adam_v2.Adam(lr=0.00001, beta_1=0.7, beta_2=0.999, epsilon=None, decay=0.0, amsgrad=False)
        # model.compile(loss=loss, optimizer=adam, metrics=metrics)

    if learning_rate_mode == 2:
        # This is the second option for learning rate reduction https://blog.csdn.net/zzc15806/article/details/79711114
        ReduceLROnPlateau(monitor='val_loss', factor=0.1, patience=10, verbose=1,
                          mode='auto', cooldown=0, min_lr=0)
        callbacks = [checkpoint, earlystop, tensorboard]#history,

    if learning_rate_mode == 3:
        lr_schedule = lambda epoch: 0.0001 * 0.4 ** epoch
        learning_rate = np.array([lr_schedule(i) for i in range(num_epochs)])
        Write_Text(save_dir + os.sep + args_info_save, f"The learning rate function is: {learning_rate}")
        changelr = LearningRateScheduler(lambda epoch: float(learning_rate[epoch]))
        callbacks = [checkpoint, earlystop, changelr,  tensorboard] #history,
        print(learning_rate[0:200])

    # 8)Start training
    print('-----------Start training-----------')
    model.fit_generator(train_loader,
                        steps_per_epoch=train_size // fit_pars['batch_size'],
                        epochs=fit_pars['num_epochs'],
                        initial_epoch=0,
                        validation_data=test_loader,
                        validation_steps=validation_size // fit_pars['batch_size'],
                        workers=fit_pars['workers'],  # Number of multiple processes
                        use_multiprocessing=fit_pars['use_multiprocessing'],  # multi-process training
                        callbacks=callbacks)
    # with open(save_dir+os.sep+'log_sgd_big_32.txt', 'w') as f:
    #     f.write(str(history.history))
    print('-----------End training-----------')


if __name__ == '__main__':

    path_wav_train = m0_para.path_wav_train
    # use_model = M2_ConvLSTM.conv1d
    # use_model = M2_ConvLSTM.bilstm
    # use_model = M2_ConvLSTM.bilstm_layer2
    # use_model = M2_ConvLSTM.dropout_3
    use_model = M2_ConvLSTM.merge_model_v4     # merge_model_v4 is the 1DCNN-BiLSTM model used in this paper
    fit_pars, feature_args = m0_para.fit_pars1_1, m0_para.feature_args
    all_label = m0_para.all_label


    def get_data(fit_pars):
        file_path_lable = path_wav_train + os.sep + fit_pars["save_file_name"]
        label_file = save_as_ob(file_path_lable, False)
        trian_file_label = label_file.loc[label_file['is_validation'] == 0]  # .iloc[:100,:]
        validation_file_label = label_file.loc[label_file['is_validation'] == 1]  # .iloc[:20,:]
        return trian_file_label, validation_file_label

    # switch path
    def get_new_path(raw, new_dict):
        parts = raw.split("\\")
        new_path = new_dict + "\\" + "\\".join(parts[-2:])
        return new_path



    trian_file_label, validation_file_label = get_data(fit_pars=fit_pars)

    trian_file_label['file_path_wav'] = trian_file_label['file_path_wav'].apply(lambda x:
                                                       get_new_path(raw=x,new_dict=r"D:\hc--document\all text\P8_medical\thesis_dataset\use_wav_raw"))  # Replace with your dataset path
    validation_file_label['file_path_wav'] = validation_file_label['file_path_wav'].apply(lambda x:
                                                       get_new_path(raw=x,new_dict=r"D:\hc--document\all text\P8_medical\thesis_dataset\use_wav_raw"))  # Replace with your dataset path
    print(trian_file_label.shape, validation_file_label.shape)

    train(
        use_model=use_model,
        trian_file=trian_file_label,
        test_file=validation_file_label,
        fit_pars=fit_pars,
        feature_args=feature_args,
        path_wav_train=path_wav_train)
