from sklearn.metrics import accuracy_score, precision_score, recall_score, f1_score, cohen_kappa_score, confusion_matrix
import matplotlib.pyplot as plt
import seaborn as sns
import pandas as pd

def evaluate_performance(data, set_name):

    y_true = data['label_score']
    y_pred = data['label_predict_score']

    acc = accuracy_score(y_true, y_pred)
    precision = precision_score(y_true, y_pred, average='macro')
    recall = recall_score(y_true, y_pred, average='macro')
    f1 = f1_score(y_true, y_pred, average='macro')
    kappa = cohen_kappa_score(y_true, y_pred)

    print(f"{set_name} Set - ACC: {acc}, Macro Precision: {precision}, Macro Recall: {recall}, Macro F1-Score: {f1}, Kappa: {kappa}")


    cm = confusion_matrix(y_true, y_pred)
    labels = ["normal", "angry", "determination", "steadiness", "tenderness", "stress"]
    plt.figure(figsize=(10, 8))
    sns.heatmap(cm, annot=True, fmt='d', cmap='Blues', xticklabels=labels, yticklabels=labels)
    plt.title(f'Confusion Matrix(1DCNN-BiLSTM)_{set_name}')
    plt.ylabel('Predicted Labels')
    plt.xlabel('True Labels')
    plt.show()

# load data
file_path = r'C:\Users\ASUS\Desktop\modified_ours(1dcnn_bilstm)_anaysis.xlsx'  # Please replace with your file path
data = pd.read_excel(file_path)
# file_path = r'C:\Users\ASUS\Desktop\data_dropout0.1_new.csv'  # Please replace with your file path
# data = pd.read_csv(file_path)
# Separate training and test sets
train_data = data[data['is_validation'] == 0]
validation_data = data[data['is_validation'] == 1]

# Evaluate training and test sets
evaluate_performance(train_data, 'Train')
evaluate_performance(validation_data, 'Test')

