#
import matplotlib.pyplot as plt
import matplotlib.ticker as ticker
import numpy as np
import pandas as pd
from sklearn import svm
from sklearn.model_selection import train_test_split
from sklearn.multiclass import OneVsRestClassifier
from sklearn.neighbors import KNeighborsClassifier
from sklearn.ensemble import RandomForestClassifier
from sklearn.linear_model import LogisticRegression
from comment_fun import save_as_ob
from sklearn.tree import DecisionTreeClassifier
from sklearn.neural_network import MLPClassifier


def get_pic(data_list, group_value, group_value_name,
            x_labels, marks, ax_set_ylable,
            ax_set_xlable, pic_save_name
            ):
    """

    """


    plt.rcParams['font.sans-serif'] = ['SimHei']  # Solve Chinese garbled characters
    plt.figure(figsize=(100, 70))
    x = np.arange(len(x_labels))

    # plt.xlabel("step", fontsize=20)
    # plt.ylabel("rate", fontsize=20)
    # plt.xticks(fontsize=18)
    # plt.yticks(fontsize=18)



    fig, ax = plt.subplots()
    if len(group_value) == 3:
        width = 0.2

        ax.bar(x - width * 2, data_list[group_value[0]], width, label=group_value_name[0], hatch=marks[0],
               color='black', edgecolor="k")
        ax.bar(x - width + 0.02, data_list[group_value[1]], width, label=group_value_name[1], hatch=marks[1], color='w',
               edgecolor="k")
        ax.bar(x + 0.04, data_list[group_value[2]], width, label=group_value_name[2], hatch=marks[2], color='w',
               edgecolor="k")
    if len(group_value) == 2:
        width = 0.3

        ax.bar(x - width / 2, data_list[group_value[0]], width, label=group_value_name[0], hatch=marks[0],
               color='black',
               edgecolor="k")
        ax.bar(x + width / 2, data_list[group_value[1]], width, label=group_value_name[1], hatch=marks[1], color='w',
               edgecolor="k")

    ax.spines['right'].set_color('none')
    ax.spines['top'].set_color('none')


    ax.tick_params(axis='both', which='major', labelsize=8)
    # ax.tick_params(axis = 'both', which = 'minor', labelsize = 4)
    plt.rcParams.update({'font.size': 9})

    ax.set_ylabel(ax_set_ylable, fontsize=10)
    ax.set_xlabel(ax_set_xlable, fontsize=10)

    ax.set_xticks(x)
    ax.set_xticklabels(x_labels)
    plt.gca().yaxis.set_major_formatter(ticker.FormatStrFormatter('%.2f'))
    plt.ylim(0.0, 1)
    ax.legend()
    plt.legend(loc=(0.855, 0.87))
    plt.gcf().subplots_adjust(bottom=0.1)
    plt.savefig(pic_save_name)
    plt.show()


# SVM model
def get_auc(data, model):
    x_train, x_test, y_train, y_test = train_test_split(data[m0_para.features_new], data[['label_score']],
                                                        test_size=0.2)

    clt = model.fit(x_train, y_train)

    print("1. Score on training set： ")

    print(clt.score(x_train, y_train))

    print("2. Score on test set： ")

    auc_all = clt.score(x_test, y_test)
    print(auc_all)

    # 4. result_predict
    result = np.argmax(clt.decision_function(x_test), axis=1)
    y_pre = pd.DataFrame({'y_predict': result})
    y_df = y_test.copy().reset_index()
    y_new = pd.concat([y_df, y_pre], axis=1)
    y_new['auc_right'] = y_new.apply(lambda x: 1 if x['label_score'] == x['y_predict'] else 0, axis=1)

    def get_right_pre(x):
        return len(x[x == 1]) / len(x)

    y_g = y_new.groupby(['label_score'])['auc_right'].agg(get_right_pre).reset_index()
    print("3. Recognition rate of various types： ")
    auc_all2 = y_new['auc_right'].agg(get_right_pre)
    print(f"4. overall recognition rate：{auc_all2} ")

    y_g['label_str'] = y_g['label_score'].apply(lambda x: m0_para.label_str[x])
    y_g['label_str_order'] = y_g['label_str'].apply(lambda x: m0_para.label_str_order[x])

    y_g2 = pd.DataFrame([{"label_str": 'average recognition rate', 'label_str_order': 6, 'auc_right': auc_all2}])

    out_df = pd.concat([y_g, y_g2])
    out_df.sort_values(by='label_str_order', inplace=True)
    return out_df

# The remaining 4 machine learning models
def get_auc_knn(data, model):
    x_train, x_test, y_train, y_test = train_test_split(data[m0_para.features_new], data[['label_score']],
                                                        test_size=0.2)
    clt = model.fit(x_train, y_train)

    print("Score on training set：", clt.score(x_train, y_train))

    print("Score on test set：", clt.score(x_test, y_test))


    probas = clt.predict_proba(x_test)
    result = np.argmax(probas, axis=1)
    y_pre = pd.DataFrame({'y_predict': result})
    y_df = y_test.copy().reset_index()
    y_new = pd.concat([y_df, y_pre], axis=1)
    y_new['auc_right'] = y_new.apply(lambda x: 1 if x['label_score'] == x['y_predict'] else 0, axis=1)

    def get_right_pre(x):
        return len(x[x == 1]) / len(x)

    y_g = y_new.groupby(['label_score'])['auc_right'].agg(get_right_pre).reset_index()
    print("3. Recognition rate of various types： ")
    auc_all2 = y_new['auc_right'].agg(get_right_pre)
    print(f"4. overall recognition rate：{auc_all2} ")

    y_g['label_str'] = y_g['label_score'].apply(lambda x: m0_para.label_str[x])
    y_g['label_str_order'] = y_g['label_str'].apply(lambda x: m0_para.label_str_order[x])

    y_g2 = pd.DataFrame([{"label_str": 'average recognition rate', 'label_str_order': 6, 'auc_right': auc_all2}])

    out_df = pd.concat([y_g, y_g2])
    out_df.sort_values(by='label_str_order', inplace=True)
    return out_df

# 1DCNN-BiLSTM model
def get_1cnn_lstm_auc():
    data2 = save_as_ob(m0_para.save_auc_predict_path, False)
    data2 = data2.loc[data2['is_validation'] == 1]

    def get_right_precent(x):
        is1 = len(x[x == 1])
        return is1 / len(x)

    data_g = data2.groupby(['new_label'])['auc_right'].agg(get_right_precent).reset_index()
    data_g['label_str'] = data_g['new_label'].apply(lambda x: m0_para.label_str2[x])
    data_g['label_str_order'] = data_g['label_str'].apply(lambda x: m0_para.label_str_order[x])

    data_g2 = data2['auc_right'].agg(get_right_precent)
    y_g2 = pd.DataFrame([{"label_str": 'average recognition rate', 'label_str_order': 6, 'auc_right': data_g2}])
    out_df = pd.concat([data_g, y_g2])
    out_df.sort_values(by='label_str_order', inplace=True)
    return out_df


if __name__ == '__main__':

    import m0_para
    data = save_as_ob(m0_para.svm_trian_data, False)

    model0 = OneVsRestClassifier(svm.SVC(kernel='linear', probability=True, random_state=1))
    model1 = svm.SVC(C=0.8, kernel='rbf', gamma=20, decision_function_shape='ovr')

    model2 = OneVsRestClassifier(svm.SVC(kernel='rbf',
                                         gamma=1,
                                         decision_function_shape='ovr',
                                         probability=True,
                                         random_state=1
                                         ))
    model_knn = OneVsRestClassifier(KNeighborsClassifier(n_neighbors=5))
    model_dt = OneVsRestClassifier(DecisionTreeClassifier(max_depth=10,
                                                          min_samples_leaf=4,
                                                          min_samples_split=8,
                                                          random_state=1))
    model_rf = OneVsRestClassifier(RandomForestClassifier(n_estimators=100,
                                                          max_depth=10,
                                                          min_samples_leaf=4,
                                                          max_features='sqrt',
                                                          bootstrap=True,
                                                          random_state=1))
    model_lr = OneVsRestClassifier(LogisticRegression(random_state=1))
    model = OneVsRestClassifier(
            MLPClassifier(hidden_layer_sizes=(100,), max_iter=400, activation='relu', solver='adam', random_state=1))
    # 3. View forecasts for each category
    auc_knn = get_auc_knn(data, model_knn)
    auc_dt = get_auc_knn(data,model_dt)
    auc_rf = get_auc_knn(data,model_rf)
    auc_lr = get_auc_knn(data,model_lr)
    auc_svm = get_auc(data, model2)
    mlp = get_auc(data, model)
    auc_cnnlstm = get_1cnn_lstm_auc()




    get_pic(data_list={'1DCNN-predict': list(auc_cnnlstm['auc_right']),
                       'svm_predict': list(auc_svm['auc_right']),
                       'knn_predict': list(auc_knn['auc_right']),
                       'dt_predict': list(auc_dt['auc_right']),
                       'rf_predict': list(auc_rf['auc_right']),
                       'lr_predict': list(auc_lr['auc_right'])},
            group_value=['1DCNN-predict', 'svm_predict', 'knn_predict', 'dt_predict', 'rf_predict', 'lr_predict'],
            group_value_name=['1DCNN-BiLSTM', "SVM", "KNN", "DT", "RF", "LR"],
            x_labels=m0_para.label_EN,
            marks=["oo", "////////", "**", "..", "xx", "\\\\\\"],
            ax_set_ylable='recognition rate',
            ax_set_xlable='Classification method',
            pic_save_name=m0_para.pic_save_name
            )