import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.metrics import accuracy_score, precision_score, recall_score, f1_score, cohen_kappa_score, confusion_matrix
from sklearn.multiclass import OneVsRestClassifier
from sklearn.neighbors import KNeighborsClassifier
from sklearn.tree import DecisionTreeClassifier
from sklearn.ensemble import RandomForestClassifier
from sklearn.linear_model import LogisticRegression

from sklearn import svm
import matplotlib.pyplot as plt
import seaborn as sns
import m0_para

# Download Data
file_path = r'C:\Users\ASUS\Desktop\发表论文\1.9的一系列作图\data_for_jiqixuexi.csv'  # Replace with your file path
data = pd.read_csv(file_path)

# Split the data set into training set and test set
features = m0_para.features_new
# X = data[features]
# y = data['label_score']
X = data[features].to_numpy()
y = data['label_score'].to_numpy()
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, stratify=y)

# Create model
# model = OneVsRestClassifier(svm.SVC(kernel='rbf', gamma=1, decision_function_shape='ovr', probability=True, random_state=1))
# model = OneVsRestClassifier(KNeighborsClassifier(n_neighbors=5))
# model = OneVsRestClassifier(DecisionTreeClassifier(max_depth=10,min_samples_leaf=4,min_samples_split=8,random_state=1))
# model = OneVsRestClassifier(RandomForestClassifier(n_estimators=100,max_depth=10,min_samples_leaf=4,max_features='sqrt',bootstrap=True,random_state=1))
model = OneVsRestClassifier(LogisticRegression(random_state=1))
# Training model
model.fit(X_train, y_train)


def evaluate_performance(X, y, set_name):
    y_pred = model.predict(X)

    # Calculate performance metrics
    acc = accuracy_score(y, y_pred)
    precision = precision_score(y, y_pred, average='macro')
    recall = recall_score(y, y_pred, average='macro')
    f1 = f1_score(y, y_pred, average='macro')
    kappa = cohen_kappa_score(y, y_pred)

    print(f"{set_name} - ACC: {acc}, Macro Precision: {precision}, Macro Recall: {recall}, Macro F1-Score: {f1}, Kappa: {kappa}")

    # Plot confusion matrix
    cm = confusion_matrix(y, y_pred)
    labels = ["normal", "angry", "determination", "steadiness", "tenderness", "stress"]
    plt.figure(figsize=(10, 8))
    sns.heatmap(cm, annot=True, fmt='d', cmap='Blues', xticklabels=labels, yticklabels=labels)
    plt.title(f'Confusion Matrix(LR)_{set_name}')
    plt.xlabel('True Labels')
    plt.ylabel('Predicted Labels')
    plt.show()

# Evaluate training and test sets
evaluate_performance(X_train, y_train, 'Train')
evaluate_performance(X_test, y_test, 'Test')
